{# vim: set ft=htmldjango sts=2 sw=2 #}
<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>django-otp demo</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.2.3/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-rbsA2VBKQhggwzxH7pPCaAqO46MgnOM80zW1RWuH61DGLwZJEdK2Kadq2F9CUG65" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.10.5/font/bootstrap-icons.css">
    <script defer src="https://cdn.jsdelivr.net/npm/bootstrap@5.2.3/dist/js/bootstrap.bundle.min.js" integrity="sha384-kenU1KFdBIe4zVF0s0G1M5b4hcpxyD9F7jL+jjXkk+Q2h455rYXK/7HAuoJl+0I4" crossorigin="anonymous"></script>
  </head>
  <body>
    {% include "navbar.html" %}

    <main class="container-fluid">
      <div class="row my-5">
        <div class="d-none d-sm-block col-sm-2 col-md-3 col-xl-4"></div>
        <div class="col-sm-8 col-md-6 col-xl-4">
          {% block main %}{% endblock %}
        </div>
        <div class="d-none d-sm-block col-sm-2 col-md-3 col-xl-4"></div>
      </div>
    </main>
  </body>
</html>
